@@ -1,5 +1,5 @@
 #!@PERL@ 
-# Copyright (c) 2009 Zmanda, Inc.  All Rights Reserved.
+# Copyright (c) 2009, 2010 Zmanda, Inc.  All Rights Reserved.
 #
 # This program is free software; you can redistribute it and/or modify it
 # under the terms of the GNU General Public License version 2 as published
@@ -19,6 +19,7 @@
 
 use lib '@amperldir@';
 use strict;
+use warnings;
 use Getopt::Long;
 
 package Amanda::Application::Amraw;
@@ -158,11 +159,6 @@ sub command_backup {
     my $self = shift;
 
     my $level = $self->{level}[0];
-    my $mesgout_fd;
-    open($mesgout_fd, '>&=3') ||
-	$self->print_to_server_and_die("Can't open mesgout_fd: $!",
-				       $Amanda::Script_App::ERROR);
-    $self->{mesgout} = $mesgout_fd;
 
     if (defined($self->{index})) {
 	$self->{'index_out'} = IO::Handle->new_from_fd(4, 'w');
@@ -207,8 +203,8 @@ sub command_backup {
 	if ($ksize < 32) {
 	    $ksize = 32;
 	}
-	print $mesgout_fd "sendbackup: size $ksize\n";
-	print $mesgout_fd "sendbackup: end\n";
+	print {$self->{mesgout}} "sendbackup: size $ksize\n";
+	print {$self->{mesgout}} "sendbackup: end\n";
     }
 
     exit 0;
