@@ -34,8 +34,8 @@
  * ...
  */
 #include "amanda.h"
-#include "version.h"
 #include "util.h"
+#include "conffile.h"
 
 int main(int argc, char **argv);
 
@@ -69,12 +69,14 @@ main(
     signal(SIGPIPE, SIG_IGN);
 
     dbopen(DBG_SUBDIR_CLIENT);
+    config_init(CONFIG_INIT_CLIENT, NULL);
+
     if (argc < 3) {
 	error(_("Need at least 3 arguments\n"));
 	/*NOTREACHED*/
     }
 
-    dbprintf(_("version %s\n"), version());
+    dbprintf(_("version %s\n"), VERSION);
 
     if (strcmp(argv[3], "--create") != 0) {
 	error(_("Can only be used to create tar archives\n"));
