@@ -33,7 +33,6 @@
 #include "sendbackup.h"
 #include "getfsent.h"
 #include "clock.h"
-#include "version.h"
 
 #define LEAF_AND_DIRS "sed -e \'\ns/^leaf[ \t]*[0-9]*[ \t]*\\.//\nt\n/^dir[ \t]/ {\ns/^dir[ \t]*[0-9]*[ \t]*\\.//\ns%$%/%\nt\n}\nd\n\'"
 
@@ -144,14 +143,16 @@ start_backup(
     char *encryptopt = skip_argument;
     char *qdisk;
     char *config;
+    level_t *alevel = (level_t *)dle->levellist->data;
+    int      level  = alevel->level;
 
-    g_snprintf(level_str, SIZEOF(level_str), "%d", GPOINTER_TO_INT(dle->level->data));
+    g_snprintf(level_str, SIZEOF(level_str), "%d", level);
 
     qdisk = quote_string(dle->disk);
-    dbprintf(_("start: %s:%s lev %d\n"), host, qdisk, GPOINTER_TO_INT(dle->level->data));
+    dbprintf(_("start: %s:%s lev %d\n"), host, qdisk, level);
 
     g_fprintf(stderr, _("%s: start [%s:%s level %d]\n"),
-	    get_pname(), host, qdisk, GPOINTER_TO_INT(dle->level->data));
+	    get_pname(), host, qdisk, level);
     amfree(qdisk);
 
     /*  apply client-side encryption here */
@@ -208,7 +209,7 @@ start_backup(
     dbprintf(_("dumping device '%s' with '%s'\n"), device, fstype);
 
 #if defined(USE_RUNDUMP) || !defined(DUMP)
-    cmd = vstralloc(amlibexecdir, "/", "rundump", versionsuffix(), NULL);
+    cmd = vstralloc(amlibexecdir, "/", "rundump", NULL);
     cmdX = cmd;
     if (g_options->config)
 	config = g_options->config;
@@ -230,7 +231,7 @@ start_backup(
 #endif							/* } */
     {
         char *progname = cmd = newvstralloc(cmd, amlibexecdir, "/", "rundump",
-					    versionsuffix(), NULL);
+					    NULL);
 	cmdX = cmd;
 	if (g_options->config)
 	    config = g_options->config;
@@ -275,7 +276,7 @@ start_backup(
     {
 #ifdef USE_RUNDUMP
         char *progname = cmd = newvstralloc(cmd, amlibexecdir, "/", "rundump",
-					    versionsuffix(), NULL);
+					    NULL);
 	cmdX = cmd;
 	if (g_options->config)
 	    config = g_options->config;
@@ -325,8 +326,7 @@ start_backup(
     if (1)
 #endif
     {
-        char *progname = cmd = newvstralloc(cmd, amlibexecdir, "/", "rundump",
-					    versionsuffix(), NULL);
+        cmd = newvstralloc(cmd, amlibexecdir, "/", "rundump", NULL);
 	cmdX = cmd;
 	if (g_options->config)
 	    config = g_options->config;
@@ -350,7 +350,7 @@ start_backup(
 			     NULL);
 	info_tapeheader(dle);
 
-	start_index(dle->create-index, dumpout, mesgf, indexf, indexcmd);
+	start_index(dle->create_index, dumpout, mesgf, indexf, indexcmd);
 
 	dumppid = pipespawn(cmd, STDIN_PIPE, 0,
 			    &dumpin, &dumpout, &mesgf, 
