@@ -162,6 +162,9 @@ am_init_feature_set(void)
 	am_add_feature(f, fe_amidxtaped_abort);
 	am_add_feature(f, fe_amrecover_correct_disk_quoting);
 	am_add_feature(f, fe_amindexd_quote_label);
+	am_add_feature(f, fe_amrecover_receive_unfiltered);
+	am_add_feature(f, fe_application_client_name);
+	am_add_feature(f, fe_script_client_name);
     }
     return f;
 }
