@@ -50,12 +50,12 @@ typedef struct dgram_s {
 
 int	dgram_bind(dgram_t *dgram, sa_family_t family, in_port_t *portp);
 void	dgram_socket(dgram_t *dgram, int sock);
-int	dgram_send_addr(struct sockaddr_storage *addr, dgram_t *dgram);
+int	dgram_send_addr(sockaddr_union *addr, dgram_t *dgram);
 ssize_t	dgram_recv(dgram_t *dgram, int timeout,
-		   struct sockaddr_storage *fromaddr);
+		   sockaddr_union *fromaddr);
 void	dgram_zero(dgram_t *dgram);
 int	dgram_cat(dgram_t *dgram, const char *fmt, ...)
-    __attribute__ ((format (printf, 2, 3)));
+     G_GNUC_PRINTF(2,3);
 void dgram_eatline(dgram_t *dgram);
 
 #endif /* ! DGRAM_H */
