@@ -140,6 +140,24 @@ localhost diskname2 $diskname {
     }
     script {
 	plugin "amlog-script"
+	single-execution yes
+	execute-where server
+	execute-on pre-host-amcheck, post-host-amcheck, pre-host-estimate, post-host-estimate, pre-host-backup, post-host-backup
+	property "logfile" "$templog"
+    }
+}
+EODLE
+$testconf->add_dle(<<EODLE);
+localhost diskname3 $diskname {
+    installcheck-test
+    program "APPLICATION"
+    application {
+	plugin "amgtar"
+	property "atime-preserve" "no"
+    }
+    script {
+	plugin "amlog-script"
+	single-execution yes
 	execute-where server
 	execute-on pre-host-amcheck, post-host-amcheck, pre-host-estimate, post-host-estimate, pre-host-backup, post-host-backup
 	property "logfile" "$templog"
@@ -152,8 +170,8 @@ unlink $templog;
 ok(run('amcheck', '-c', 'TESTCONF'), "amcheck runs successfully for server scripts.");
 
 verify_log("amcheck invokes correct script commands",
-    "check TESTCONF pre-host-amcheck server localhost diskname2 $diskname",
-    "check TESTCONF post-host-amcheck server localhost diskname2 $diskname",
+    "check TESTCONF pre-host-amcheck server localhost diskname3 $diskname",
+    "check TESTCONF post-host-amcheck server localhost diskname3 $diskname",
 );
 
 unlink $templog;
@@ -161,10 +179,10 @@ ok(run('amdump', 'TESTCONF'), "amdump runs successfully for server scripts.")
     or amdump_diag();
 
 verify_log("amdump invokes correct script commands",
-    "estimate TESTCONF pre-host-estimate server localhost diskname2 $diskname 0",
-    "estimate TESTCONF post-host-estimate server localhost diskname2 $diskname 0",
-    "backup TESTCONF pre-host-backup server localhost diskname2 $diskname",
-    "backup TESTCONF post-host-backup server localhost diskname2 $diskname",
+    "estimate TESTCONF pre-host-estimate server localhost diskname3 $diskname",
+    "estimate TESTCONF post-host-estimate server localhost diskname3 $diskname",
+    "backup TESTCONF pre-host-backup server localhost diskname3 $diskname",
+    "backup TESTCONF post-host-backup server localhost diskname3 $diskname",
 );
 
 unlink $templog;
