@@ -32,6 +32,7 @@
 #include "amanda.h"
 #include "util.h"
 #include "holding.h"
+#include "diskfile.h"
 #include "fileheader.h"
 #include "logfile.h"
 
@@ -516,7 +517,6 @@ holding_get_files_for_flush(
     GSList *file_list, *file_elt;
     GSList *date;
     int date_matches;
-    disk_t *dp;
     dumpfile_t file;
     GSList *result_list = NULL;
 
@@ -551,15 +551,6 @@ holding_get_files_for_flush(
             continue;
 	}
 
-        /* check that the hostname and disk are in the disklist */
-        dp = lookup_disk(file.name, file.disk);
-        if (dp == NULL) {
-	    dbprintf(_("%s: disk %s:%s not in database, skipping it."),
-                        (char *)file_elt->data, file.name, file.disk);
-	    dumpfile_free_data(&file);
-            continue;
-        }
-
         /* passed all tests -- we'll flush this file */
         result_list = g_slist_insert_sorted(result_list, 
 	    stralloc(file_elt->data), 
@@ -567,7 +558,7 @@ holding_get_files_for_flush(
 	dumpfile_free_data(&file);
     }
 
-    if (file_list) g_slist_free_full(file_list);
+    if (file_list) slist_free_full(file_list, g_free);
 
     return result_list;
 }
@@ -593,7 +584,7 @@ holding_get_all_datestamps(void)
 	dumpfile_free_data(&dfile);
     }
 
-    g_slist_free_full(all_files);
+    slist_free_full(all_files, g_free);
 
     return datestamps;
 }
@@ -952,9 +943,11 @@ rename_tmp_holding(
 		dumpfile_free_data(&file);
 		amfree(filename);
 		amfree(filename_tmp);
+		free(header);
 		close(fd);
 		return 0;
 	    }
+	    free(header);
 	    close(fd);
 	}
 	filename = newstralloc(filename, file.cont_filename);
